<?php

// Nine Letter Puzzle search (e.g. http://imgur.com/gallery/X32AvSX )

error_reporting(E_ALL);
ini_set('display_errors', 'On');

$children = [];
$roots = [];

// 0. Load dictionary

$dictionary = explode("\n", file_get_contents('american-english'));
unset($dictionary[count($dictionary)-1]); // the last entry is the empty-string
for ($i = 0, $e = count($dictionary); $i !== $e; ++$i) {
	$dictionary[$i] = mb_strtolower($dictionary[$i], 'UTF-8');
}

shuffle($dictionary); // Doesn't matter, we always get the same best results - but it's nice to vary the journey

$inverted_dictionary = [];
foreach($dictionary as $i => $word) {
	$inverted_dictionary[$word] = $i;
}

// 1. Find roots, and children of each word

echo "Finding roots and children...\n";

foreach($dictionary as $word) {
	
	$parents = [];
	for($i = 0; $i < strlen($word); ++$i) {
		$possible_parent = substr($word, 0, $i).substr($word, $i+1);
		if (array_key_exists($possible_parent, $inverted_dictionary)) {
			$parents[] = $possible_parent;
		}
	}
	
	if (count($parents)) {
		foreach($parents as $parent) {
			if (! isset($children[$parent])) {
				$children[$parent] = [];
			}
			$children[$parent][] = $word;
		}
		
	} else {
		$roots[] = $word;
		
	}
	
	// Progress display
	if (rand() % 200 == 0) {
		echo "- Progress: $word \n";
	}
	
}

// 2. From each root, walk depth-first

echo "- Walking tree...\n";

$roots = ['i', 'a']; // ignore all others

$walker = function($word, $parent_chain=[]) use (&$walker, $children) {
	
	// Display any guess above 9
	if (count($parent_chain) >= 9) {
		echo sprintf("%-2d", count($parent_chain))." - ".json_encode($parent_chain)." \n";
	}
	
	if (isset($children[$word])) {
		
		$chain = $parent_chain;
		$chain[] = $word;
		
		foreach($children[$word] as $child_word) {
			$walker($child_word, $chain);
		}
	}
};

foreach($roots as $root) {
	$walker($root);
}
